from PyQt5.QtWidgets import QWidget
from PyQt5.QtCore import pyqtSignal

from .modifier_picker_ui import Ui_ModifierCheckboxes

MODIFIERS = ('ctrl', 'shift', 'alt', 'alt_gr')

class ModifierCheckboxes(QWidget):
    """ A widget with 4 checkboxes for each of the 4 keyboard modifiers """
    values_changed = pyqtSignal(tuple)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.ui = Ui_ModifierCheckboxes()
        self.ui.setupUi(self)

    def _get_values(self):
        """
        :return: A list of four bool in the same order than MODIFIERS showing
                 the check state of the matching QCheckBox
        """
        return [chkbx.isChecked() for chkbx in self._get_checkbox_list()]

    def _get_checkbox_list(self):
        """
        :return: The list of QCheckBoxes of this widget, in the same order
                 than MODIFIERS
        """
        return [getattr(self.ui, name+'CheckBox') for name in MODIFIERS]

    def values(self):
        """ :retrun: The subset of MODIFIERS which are actually checked """
        return [MODIFIERS[i] for i in range(len(MODIFIERS)) if self._get_values()[i]]

    def set_values(self, values):
        """ :type values: list of str to pick in MODIFIERS """
        original_values = self._get_values()
        for m, widget in zip(MODIFIERS, self._get_checkbox_list()):
            widget.setChecked(m in values)

        if original_values != self._get_values():
            self.values_changed.emit()



import logging as log

from PyQt5.QtCore import Qt, QCoreApplication, QObject, pyqtSignal, pyqtSlot
from PyQt5.QtCore import QVariant

from pynput.keyboard import Key

from .list import ListModel
from .data.events import Event, MouseClick, MouseDoubleClick, \
    TextEntered, KeyPressed

class EventList(QObject):
    """
    Class managing a collection of user input events.
    All items in the list of events must subclass Event
    """
    list_changed = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)

        self.events = []
        self.offset = 0

    #region## Elements management
    def push(self, event):
        """
        Adds an event to the list. Merge with the previous event
        when possible.
        :param event: The event to add
        :type event: Any subclass of Event
        """
        #pylint: disable=unidiomatic-typecheck
        if not self.events:
            # ensures no other branch is processed => self.events[-1] can
            # be called safely in other branches
            pass
        elif isinstance(event, TextEntered):
            if isinstance(self.events[self.offset-1], TextEntered):
                self.events[self.offset-1].text += str(event.text)
                event = None
        elif type(event) == KeyPressed:
            if isinstance(self.events[self.offset-1], TextEntered):
                if event.key == Key.space.name:
                    self.events[self.offset-1].text += str(' ')
                    event = None
                elif event.key == Key.backspace.name:
                    self.events[self.offset-1].text = self.events[self.offset-1].text[:-1]
                    event = None
        # type() is used on purpose here, ie we want the real type
        elif type(event) == MouseClick:
            if type(self.events[self.offset-1]) == MouseClick:
                if abs(event.time - self.events[self.offset-1].time) < Event.DOUBLE_CLICK_THRESHOLD:
                    self.events[self.offset-1] = MouseDoubleClick(event.x, event.y)
                    event = None

        if event:
            self.events.insert(self.offset, event)
            self.offset += 1
            log.debug('New offset = %d', self.offset)

        log.debug('Last event: %s. New list size: %d',
            self.events[self.offset-1].__class__.__name__,
            self.size())

        self.list_changed.emit()

    def pop(self):
        """ Removes and returns the last event in the list """
        event = None
        if self.size() > 0:
            self.offset -= 1
            event = self.events[self.offset]
            del self.events[self.offset]
            self.list_changed.emit()
        return event

    @pyqtSlot()
    def clear(self):
        """ Removes all the elements from the list """
        del self.events[:]
        self.list_changed.emit()
    def insert(self, index, event_type):
        if index < 0:
            self.events.append(Event.create(event_type))
        else:
            self.events.insert(index, Event.create(event_type))
        self.list_changed.emit()

    @pyqtSlot(Event)
    def remove(self, event):
        """ Remove event from list """
        if event in self.events:
            self.events.remove(event)
            self.list_changed.emit()
            return True
        return False

    def move(self, event, direction):
        """
        Change event position in list
        :param direction: +1 to move down, -1 to move up
        """
        if event in self.events:
            index = self.events.index(event)
            self.events[index+direction], self.events[index] = self.events[index], self.events[index+direction]
            self.list_changed.emit()
            return True
        return False


    def replace(self, old, new):
        """ Replaces an event in the list by another """
        if old in self.events:
            self.events[self.events.index(old)] = new
            self.list_changed.emit()
            return True
        return False

    def size(self):
        """ :return: The number of events """
        return len(self.events)
    #endregion##

    def get_serializable(self):
        """ :return: this object as a dict """
        return {
            'events': list(map(lambda l: l.get_serializable(), self.events))
        }


class EventListModel(ListModel):
    """ Model for displaying an EventList in a QTableView """
    HEADERS = [
        QCoreApplication.translate('EventListModel', 'Type'),
        QCoreApplication.translate('EventListModel', 'Description')
    ]

    def __init__(self, data, parent=None):
        """
        :type data: EventList
        """
        super().__init__(data, parent)
        self._last_col = len(EventListModel.HEADERS)-1

    def columnCount(self, parent):
        #pylint: disable=unused-argument, no-self-use
        """ There's 2 columns in the table: Type and Description string"""
        return len(EventListModel.HEADERS)

    def data(self, index, role=Qt.DisplayRole):
        """
        Returns the data for the specified cell for the specified role
        :param index: The table cell
        :type index: QModelIndex
        :param role: The Qt role of the returned value (only DisplayRole is implemented)
        """
        value = QVariant()
        if role == Qt.DisplayRole:
            event = self._data.events[index.row()]
            col = index.column()
            if col == 0:
                value = event.__class__.__name__
            elif col == 1:
                value = str(event)

        return value

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        #pylint: disable=no-self-use
        """ The table column titles """
        if role == Qt.DisplayRole:
            if orientation == Qt.Horizontal:
                return EventListModel.HEADERS[section]
            return section
        return QVariant()
    #endregion##



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './snitch/ui/settings.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SettingsDialog(object):
    def setupUi(self, SettingsDialog):
        SettingsDialog.setObjectName("SettingsDialog")
        SettingsDialog.resize(450, 250)
        self.verticalLayout = QtWidgets.QVBoxLayout(SettingsDialog)
        self.verticalLayout.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBoxGeneral = QtWidgets.QGroupBox(SettingsDialog)
        self.groupBoxGeneral.setObjectName("groupBoxGeneral")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBoxGeneral)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.groupBoxGeneral)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(102, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 1, 1, 1)
        self.valueMaxRecent = QtWidgets.QSpinBox(self.groupBoxGeneral)
        self.valueMaxRecent.setMinimumSize(QtCore.QSize(60, 0))
        self.valueMaxRecent.setMinimum(1)
        self.valueMaxRecent.setMaximum(25)
        self.valueMaxRecent.setObjectName("valueMaxRecent")
        self.gridLayout.addWidget(self.valueMaxRecent, 0, 2, 1, 1)
        self.verticalLayout.addWidget(self.groupBoxGeneral)
        self.groupBoxRecording = QtWidgets.QGroupBox(SettingsDialog)
        self.groupBoxRecording.setObjectName("groupBoxRecording")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxRecording)
        self.formLayout.setObjectName("formLayout")
        self.checkBoxMinimizeOnRecord = QtWidgets.QCheckBox(self.groupBoxRecording)
        self.checkBoxMinimizeOnRecord.setObjectName("checkBoxMinimizeOnRecord")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.checkBoxMinimizeOnRecord)
        self.verticalLayout.addWidget(self.groupBoxRecording)
        self.groupBoxPlayback = QtWidgets.QGroupBox(SettingsDialog)
        self.groupBoxPlayback.setObjectName("groupBoxPlayback")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBoxPlayback)
        self.gridLayout_2.setObjectName("gridLayout_2")
        spacerItem1 = QtWidgets.QSpacerItem(80, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_2.addItem(spacerItem1, 0, 1, 1, 1)
        self.labelPlaybackDelay = QtWidgets.QLabel(self.groupBoxPlayback)
        self.labelPlaybackDelay.setObjectName("labelPlaybackDelay")
        self.gridLayout_2.addWidget(self.labelPlaybackDelay, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBoxPlayback)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 3, 0, 1, 1)
        self.valuePlaybackDelay = QtWidgets.QDoubleSpinBox(self.groupBoxPlayback)
        self.valuePlaybackDelay.setMinimumSize(QtCore.QSize(60, 0))
        self.valuePlaybackDelay.setDecimals(1)
        self.valuePlaybackDelay.setMinimum(-1.0)
        self.valuePlaybackDelay.setSingleStep(0.1)
        self.valuePlaybackDelay.setProperty("value", 0.2)
        self.valuePlaybackDelay.setObjectName("valuePlaybackDelay")
        self.gridLayout_2.addWidget(self.valuePlaybackDelay, 0, 2, 1, 1)
        self.valueDiffThreshold = QtWidgets.QSpinBox(self.groupBoxPlayback)
        self.valueDiffThreshold.setEnabled(True)
        self.valueDiffThreshold.setMaximum(100)
        self.valueDiffThreshold.setProperty("value", 100)
        self.valueDiffThreshold.setObjectName("valueDiffThreshold")
        self.gridLayout_2.addWidget(self.valueDiffThreshold, 3, 2, 1, 1)
        self.valueDiffThresholdSlider = QtWidgets.QSlider(self.groupBoxPlayback)
        self.valueDiffThresholdSlider.setMinimumSize(QtCore.QSize(150, 0))
        self.valueDiffThresholdSlider.setMinimum(50)
        self.valueDiffThresholdSlider.setMaximum(100)
        self.valueDiffThresholdSlider.setProperty("value", 100)
        self.valueDiffThresholdSlider.setOrientation(QtCore.Qt.Horizontal)
        self.valueDiffThresholdSlider.setTickPosition(QtWidgets.QSlider.TicksAbove)
        self.valueDiffThresholdSlider.setObjectName("valueDiffThresholdSlider")
        self.gridLayout_2.addWidget(self.valueDiffThresholdSlider, 3, 1, 1, 1)
        self.verticalLayout.addWidget(self.groupBoxPlayback)
        self.buttonBox = QtWidgets.QDialogButtonBox(SettingsDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SettingsDialog)
        self.buttonBox.accepted.connect(SettingsDialog.accept)
        self.buttonBox.rejected.connect(SettingsDialog.reject)
        self.valueDiffThresholdSlider.valueChanged['int'].connect(self.valueDiffThreshold.setValue)
        self.valueDiffThreshold.valueChanged['int'].connect(self.valueDiffThresholdSlider.setValue)
        QtCore.QMetaObject.connectSlotsByName(SettingsDialog)

    def retranslateUi(self, SettingsDialog):
        _translate = QtCore.QCoreApplication.translate
        SettingsDialog.setWindowTitle(_translate("SettingsDialog", "Snitch preferences"))
        self.groupBoxGeneral.setTitle(_translate("SettingsDialog", "General"))
        self.label.setText(_translate("SettingsDialog", "Max number of recent files: "))
        self.groupBoxRecording.setTitle(_translate("SettingsDialog", "Event recordinq"))
        self.checkBoxMinimizeOnRecord.setText(_translate("SettingsDialog", "Minimize when recording"))
        self.groupBoxPlayback.setTitle(_translate("SettingsDialog", "Event playback"))
        self.labelPlaybackDelay.setText(_translate("SettingsDialog", "Delay between replayed events:"))
        self.label_2.setText(_translate("SettingsDialog", "Diff threshold:"))
        self.valuePlaybackDelay.setSuffix(_translate("SettingsDialog", "s"))
        self.valueDiffThreshold.setSuffix(_translate("SettingsDialog", "%"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './snitch/ui/list_manager.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_EventMgmtButtons(object):
    def setupUi(self, EventMgmtButtons):
        EventMgmtButtons.setObjectName("EventMgmtButtons")
        EventMgmtButtons.resize(206, 71)
        self.horizontalLayout = QtWidgets.QHBoxLayout(EventMgmtButtons)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.buttonAdd = QtWidgets.QToolButton(EventMgmtButtons)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/controls/ui/icons/add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.buttonAdd.setIcon(icon)
        self.buttonAdd.setIconSize(QtCore.QSize(24, 24))
        self.buttonAdd.setPopupMode(QtWidgets.QToolButton.InstantPopup)
        self.buttonAdd.setObjectName("buttonAdd")
        self.horizontalLayout.addWidget(self.buttonAdd)
        self.buttonDel = QtWidgets.QPushButton(EventMgmtButtons)
        self.buttonDel.setEnabled(False)
        self.buttonDel.setMinimumSize(QtCore.QSize(32, 32))
        self.buttonDel.setMaximumSize(QtCore.QSize(32, 32))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/controls/ui/icons/remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.buttonDel.setIcon(icon1)
        self.buttonDel.setIconSize(QtCore.QSize(24, 24))
        self.buttonDel.setObjectName("buttonDel")
        self.horizontalLayout.addWidget(self.buttonDel)
        spacerItem = QtWidgets.QSpacerItem(51, 0, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.buttonDown = QtWidgets.QPushButton(EventMgmtButtons)
        self.buttonDown.setEnabled(False)
        self.buttonDown.setMinimumSize(QtCore.QSize(32, 32))
        self.buttonDown.setMaximumSize(QtCore.QSize(32, 32))
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/controls/ui/icons/down.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.buttonDown.setIcon(icon2)
        self.buttonDown.setIconSize(QtCore.QSize(24, 24))
        self.buttonDown.setObjectName("buttonDown")
        self.horizontalLayout.addWidget(self.buttonDown)
        self.buttonUp = QtWidgets.QPushButton(EventMgmtButtons)
        self.buttonUp.setEnabled(False)
        self.buttonUp.setMinimumSize(QtCore.QSize(32, 32))
        self.buttonUp.setMaximumSize(QtCore.QSize(32, 32))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/controls/ui/icons/up.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.buttonUp.setIcon(icon3)
        self.buttonUp.setIconSize(QtCore.QSize(24, 24))
        self.buttonUp.setObjectName("buttonUp")
        self.horizontalLayout.addWidget(self.buttonUp)

        self.retranslateUi(EventMgmtButtons)
        QtCore.QMetaObject.connectSlotsByName(EventMgmtButtons)

    def retranslateUi(self, EventMgmtButtons):
        _translate = QtCore.QCoreApplication.translate
        EventMgmtButtons.setWindowTitle(_translate("EventMgmtButtons", "Form"))
        self.buttonDel.setToolTip(_translate("EventMgmtButtons", "Remove selected event"))
        self.buttonDown.setToolTip(_translate("EventMgmtButtons", "Move event down"))
        self.buttonUp.setToolTip(_translate("EventMgmtButtons", "Move event up"))
from snitch import assets_rc



from PyQt5.QtCore import QObject, QTimer, pyqtSignal, pyqtSlot, Qt
from PyQt5.QtWidgets import QFrame, QLabel

import pyautogui


class MouseTracker(QObject):
    """ Object for tracking mouse position """
    position_changed = pyqtSignal((str,), (int, int))

    def __init__(self, parent=None):
        super().__init__(parent)

        self._mousepos_timer = QTimer(self)
        self._mousepos_timer.timeout.connect(self._update_mouse_position)
        self._mousepos_timer.start(50)

    @pyqtSlot()
    def _update_mouse_position(self):
        self.position_changed[str].emit('x={} ; y={}'.format(*pyautogui.position()))
        self.position_changed[int, int].emit(*list(pyautogui.position()))

    def get_widget(self, parent=None):
        """ Provides a QLabel connected to this object showing the mouse position """
        label = QLabel(parent)
        label.setMinimumWidth(120)
        label.setFrameShape(QFrame.Panel)
        label.setFrameShadow(QFrame.Sunken)
        label.setAlignment(Qt.AlignCenter)
        self.position_changed.connect(label.setText)
        return label



#!/usr/bin/python3
"""
Takes one or more test case files created with Snitch, extracts all the
screenshots from them, and saves the images in the current directory.
"""

import sys
import os
import json


from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QPixmap
from PyQt5.QtCore import QByteArray, qUncompress


def save_base64_to_png(base64_encoded_image, filename):
    if base64_encoded_image:
        img = QPixmap()
        if img.loadFromData(qUncompress(QByteArray.fromBase64(bytes(base64_encoded_image, 'utf8'))), 'PNG'):
            img.save(filename)

def main():
    APP = QApplication(sys.argv)
    if len(sys.argv) < 2:
        print('You must specify at least one file to read')
        sys.exit(1)

    for snitch_file in sys.argv[1:]:
        if os.path.isfile(snitch_file):
            name = os.path.splitext(os.path.basename(snitch_file))[0]
            for i, cap in enumerate(json.load(open(snitch_file, 'r'))['screenshots']):
                save_base64_to_png(cap['image'], '{}_image{:03d}_original.png'.format(name, i))
                save_base64_to_png(cap['result'], '{}_image{:03d}_result.png'.format(name, i))
        else:
            print('File does not exist: {}'.format(snitch_file))

    APP.quit()

if __name__ == '__main__':
    main()



""" File format """
import logging as log
import json
import pickle
import os

import pyautogui
from PyQt5.QtCore import QObject

from ... import FILE_VERSION
from .events import Event
from ..eventlist import EventList
from .screenshot import Screenshot
from ..snaplist import SnapList

class TestCase(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)

        self._file_version = FILE_VERSION
        self.name = 'test_case'
        self._screen = pyautogui.size()

        self.event_list = EventList(self)
        self.snap_list = SnapList(self)

    def save(self, filename, export_module=json):
        """
        Saves the event recording into a file
        :param filename: The name of the file
        :param export_module: Either pickle or json
        """
        log.info('Saving %d events into "%s"', self.event_list.size(), filename)
        self.name = os.path.splitext(os.path.basename(filename))[0]
        try:
            file_content = {
                'version': self._file_version,
                'screen': pyautogui.size(),
                **self.event_list.get_serializable(),
                **self.snap_list.get_serializable()
            }
            if export_module == json:
                with open(filename, 'w') as record:
                    log.debug(file_content)
                    json.dump(file_content, record, indent=4, ensure_ascii=False)
            else:
                with open(filename, 'bw') as record:
                    pickle.dump(file_content, record)
        except OSError as ex:
            log.error(str(ex))

    def load(self, filename, import_module=json):
        """
        Loads the event recording from a file
        :param filename: The name of the file
        :param import_module: Either pickle or json
        """
        try:
            if import_module == json:
                with open(filename, 'r') as record:
                    data = json.load(record)
                    self._build_list_from_dict(data)
            elif import_module == pickle:
                with open(filename, 'r') as record:
                    file_content = pickle.load(record)
                    self._file_version = file_content['version']
                    self.event_list = file_content['events']
                    self.snap_list = file_content['screenshots']
            else:
                log.error('Unsuported file format: %s', import_module)

            self.name = os.path.splitext(os.path.basename(filename))[0]
            log.info('Loaded %d events from "%s"', self.event_list.size(), filename)
        except OSError as ex:
            log.error(str(ex))

    def _build_list_from_dict(self, event_list_dict):
        """
        :param event_list_dict: dictionnary of the JSON decoded object
        """
        try:
            self._file_version = event_list_dict['version']
            for event_item in event_list_dict['events']:
                event = Event.create(event_item['type'])
                for key, val in event_item.items():
                    event.__dict__[key] = val
                self.event_list.push(event)
            self.event_list.list_changed.emit()
            for snap_item in event_list_dict['screenshots']:
                snap = Screenshot()
                for key, val in snap_item.items():
                    snap.__dict__[key] = val
                self.snap_list.push(snap)
            self.snap_list.list_changed.emit()

        except (KeyError, IndexError) as err:
            log.error("Can't decode file: %s", err)



from PyQt5.QtWidgets import QLabel
from PyQt5.QtCore import Qt, QSize

class AspectRatioPixmapLabel(QLabel):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setMinimumSize(1, 1)
        self.setScaledContents(False)
        self._pixmap = None

    def setPixmap(self, pixmap):
        self._pixmap = pixmap
        super().setPixmap(self._scaled_pixmap())

    def heightForWidth(self, width):
        if self._pixmap:
            return self._pixmap.height() / self._pixmap.width() * width
        return self.height()

    def sizeHint(self):
        return QSize(self.width(), self.heightForWidth(self.width()))

    def resizeEvent(self, event):
        # pylint: disable=unused-argument
        if self._pixmap:
            super().setPixmap(self._scaled_pixmap())

    def _scaled_pixmap(self):
        return self._pixmap.scaled(self.size(), Qt.KeepAspectRatio, Qt.SmoothTransformation)



#!/usr/bin/python3
"""
Takes a PNG image as parameter and prints the OCR result to the standard output.
"""

import logging as log

import numpy as np
import pytesseract
import cv2

DEFAULT_SCALE_FACTOR = 2
DEFAULT_INTERPOLATION = cv2.INTER_CUBIC


def ocr(image, imgoutput=None, txtoutput=None, lang='eng', character_subset=''):
    """
    :param image: the cv2 image object to perform OCR on (output of cv2.imread)
    :param imgoutput: the filename to save the image (default doesn’t save file)
    :param txtoutput: the filename to save the ocr’ed text (default doesn’t save file)
    :param character_subset: a string containing the characters to
    :return: the output text
    """
    if imgoutput:
        cv2.imwrite(imgoutput, image)
    cfg = ''
    if character_subset:
        cfg = "-c tessedit_char_whitelist='{}'".format(character_subset)

    text = pytesseract.image_to_string(image, lang=lang, config=cfg)

    if txtoutput:
        with open(txtoutput, 'w', encoding='utf-8') as f:
            f.write(text)

    return text


def process_image(image, factor=DEFAULT_SCALE_FACTOR, interpolation=DEFAULT_INTERPOLATION, lang='eng', character_subset=''):
    """
    :param image: the image to use as input, or a string of the path to the image
    :param factor: the scale factor: 2 means an image 2 times bigger
    :param interpolation: the interpolation method to use when scaling the image
    :param lang: the expected language (as a 3 letter code) of the recognized text,
        multiple language can be specified using ’+’ as delimiter
    :return: the output text
    """
    # Read image using opencv
    if isinstance(image, np.ndarray):
        img = image
    elif isinstance(image, str):
        img = cv2.imread(image)
    else:
        raise TypeError('«{}» is not a supported image type.'.format(type(image)))

    # Rescale the image, if needed.
    img = cv2.resize(img, None, fx=factor, fy=factor, interpolation=interpolation)

    # Convert to gray
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # Apply dilation and erosion to remove some noise (not useful here)
    #kernel = np.ones((1, 1), np.uint8)
    #img = cv2.erode(img, kernel, iterations=2)
    #img = cv2.dilate(img, kernel, iterations=1)

    # Apply threshold to get image with only black and white
    img = cv2.threshold(img, 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1]

    result = ocr(img, lang=lang, character_subset=character_subset)

    log.info('Image recognition result\n%s\n', result)
    return result



#!/usr/bin/python3
"""
Takes a PNG image as parameter and prints the OCR result to the standard output.
"""
import os
import sys

from PIL import Image
import cv2
import pytesseract

from snitch.ocr import ocr, DEFAULT_SCALE_FACTOR, DEFAULT_INTERPOLATION


def test_methods(img_path, factor=DEFAULT_SCALE_FACTOR, interpolation=DEFAULT_INTERPOLATION, lang='eng'):
    # Read image using opencv
    img = cv2.imread(img_path)
    # Extract the file name without the file extension
    file_name = os.path.basename(os.path.splitext(img_path)[0]+'_x{}'.format(factor))
    # Create a directory for outputs
    output_path = os.path.join(os.path.dirname(img_path), file_name)
    if not os.path.exists(output_path):
        os.makedirs(output_path)

    # Rescale the image, if needed.
    img = cv2.resize(img, None, fx=factor, fy=factor, interpolation=interpolation)

    # Convert to gray
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    # Apply dilation and erosion to remove some noise (not useful here)
    #kernel = np.ones((1, 1), np.uint8)
    #img = cv2.erode(img, kernel, iterations=2)
    #img = cv2.dilate(img, kernel, iterations=1)

    # Apply threshold to get image with only black and white
    # pylint: disable=line-too-long
    images = {
        ('threshold', 'gaussian', '9', 'binary')     : cv2.threshold(cv2.GaussianBlur(img, (9, 9), 0), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'gaussian', '7', 'binary')     : cv2.threshold(cv2.GaussianBlur(img, (7, 7), 0), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'gaussian', '5', 'binary')     : cv2.threshold(cv2.GaussianBlur(img, (5, 5), 0), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'gaussian', '3', 'binary')     : cv2.threshold(cv2.GaussianBlur(img, (3, 3), 0), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'none',     '0', 'binary')     : cv2.threshold(img, 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'gaussian', '9', 'tozero')     : cv2.threshold(cv2.GaussianBlur(img, (9, 9), 0), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'gaussian', '7', 'tozero')     : cv2.threshold(cv2.GaussianBlur(img, (7, 7), 0), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'gaussian', '5', 'tozero')     : cv2.threshold(cv2.GaussianBlur(img, (5, 5), 0), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'gaussian', '3', 'tozero')     : cv2.threshold(cv2.GaussianBlur(img, (3, 3), 0), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'none',     '0', 'tozero')     : cv2.threshold(img, 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'gaussian', '9', 'binary_otsu'): cv2.threshold(cv2.GaussianBlur(img, (9, 9), 0), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '7', 'binary_otsu'): cv2.threshold(cv2.GaussianBlur(img, (7, 7), 0), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '5', 'binary_otsu'): cv2.threshold(cv2.GaussianBlur(img, (5, 5), 0), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '3', 'binary_otsu'): cv2.threshold(cv2.GaussianBlur(img, (3, 3), 0), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'none',     '0', 'binary_otsu'): cv2.threshold(img, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '9', 'tozero_otsu'): cv2.threshold(cv2.GaussianBlur(img, (9, 9), 0), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '7', 'tozero_otsu'): cv2.threshold(cv2.GaussianBlur(img, (7, 7), 0), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '5', 'tozero_otsu'): cv2.threshold(cv2.GaussianBlur(img, (5, 5), 0), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'gaussian', '3', 'tozero_otsu'): cv2.threshold(cv2.GaussianBlur(img, (3, 3), 0), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'none',     '0', 'tozero_otsu'): cv2.threshold(img, 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('adaptive',  'gaussian', '9', 'binary_otsu'): cv2.adaptiveThreshold(cv2.GaussianBlur(img, (9, 9), 0), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'gaussian', '7', 'binary')     : cv2.adaptiveThreshold(cv2.GaussianBlur(img, (7, 7), 0), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'gaussian', '5', 'binary')     : cv2.adaptiveThreshold(cv2.GaussianBlur(img, (5, 5), 0), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'gaussian', '3', 'binary')     : cv2.adaptiveThreshold(cv2.GaussianBlur(img, (3, 3), 0), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'none',     '0', 'binary')     : cv2.adaptiveThreshold(img, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('threshold', 'median',   '9', 'binary')     : cv2.threshold(cv2.medianBlur(img, 9), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'median',   '7', 'binary')     : cv2.threshold(cv2.medianBlur(img, 7), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'median',   '5', 'binary')     : cv2.threshold(cv2.medianBlur(img, 5), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'median',   '3', 'binary')     : cv2.threshold(cv2.medianBlur(img, 3), 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'none',     '0', 'binary')     : cv2.threshold(img, 0, 255, cv2.THRESH_BINARY)[1],
        ('threshold', 'median',   '9', 'tozero')     : cv2.threshold(cv2.medianBlur(img, 9), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'median',   '7', 'tozero')     : cv2.threshold(cv2.medianBlur(img, 7), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'median',   '5', 'tozero')     : cv2.threshold(cv2.medianBlur(img, 5), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'median',   '3', 'tozero')     : cv2.threshold(cv2.medianBlur(img, 3), 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'none',     '0', 'tozero')     : cv2.threshold(img, 0, 255, cv2.THRESH_TOZERO)[1],
        ('threshold', 'median',   '9', 'binary_otsu'): cv2.threshold(cv2.medianBlur(img, 9), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '7', 'binary_otsu'): cv2.threshold(cv2.medianBlur(img, 7), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '5', 'binary_otsu'): cv2.threshold(cv2.medianBlur(img, 5), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '3', 'binary_otsu'): cv2.threshold(cv2.medianBlur(img, 3), 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'none',     '0', 'binary_otsu'): cv2.threshold(img, 0, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '9', 'tozero_otsu'): cv2.threshold(cv2.medianBlur(img, 9), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '7', 'tozero_otsu'): cv2.threshold(cv2.medianBlur(img, 7), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '5', 'tozero_otsu'): cv2.threshold(cv2.medianBlur(img, 5), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'median',   '3', 'tozero_otsu'): cv2.threshold(cv2.medianBlur(img, 3), 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('threshold', 'none',     '0', 'tozero_otsu'): cv2.threshold(img, 0, 255, cv2.THRESH_TOZERO | cv2.THRESH_OTSU)[1],
        ('adaptive',  'median',   '9', 'binary_otsu'): cv2.adaptiveThreshold(cv2.medianBlur(img, 9), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'median',   '7', 'binary')     : cv2.adaptiveThreshold(cv2.medianBlur(img, 7), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'median',   '5', 'binary')     : cv2.adaptiveThreshold(cv2.medianBlur(img, 5), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'median',   '3', 'binary')     : cv2.adaptiveThreshold(cv2.medianBlur(img, 3), 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
        ('adaptive',  'none',     '0', 'binary')     : cv2.adaptiveThreshold(img, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 31, 2),
    }

    # for each threshold method, save the preprocessed image and the ocr’ed text
    for key, img in images.items():
        # Save the filtered image in the output directory
        options_str = '{}_{}'.format(file_name, '_'.join(key))
        save_path = os.path.join(output_path, options_str)

        result = ocr(img, imgoutput=save_path+'png', txtoutput=save_path+'txt')

        print('{} {} //'. format('/'*42, options_str))
        print(result)


def main():
    if len(sys.argv) < 2:
        print("Missing required parameter")
        sys.exit(1)

    image = sys.argv[1]
    #for i in range(1, 5):
    #    test_methods(image, factor=i)
    test_methods(image, 2, interpolation=cv2.INTER_CUBIC, lang='eng+equ')


if __name__ == '__main__':
    main()



from os.path import splitext

from PyQt5.QtCore import pyqtSlot
from PyQt5.QtWidgets import QWidget, QFileDialog

from .diff_ui import Ui_Diff

from ..settings import CFG


class Diff(QWidget):
    """ Main window for the app """
    def __init__(self, reference_image, result_image, diff_image, reference_ocr=None, result_ocr=None, parent=None):
        """
        :type reference_image: QPixmap
        :type result_image: QPixmap
        :type diff_image: QPixmap
        """
        super().__init__(parent)

        self._ui = Ui_Diff()
        self._ui.setupUi(self)

        self._ui.referenceImageLabel.setPixmap(reference_image)
        self._ui.resultImageLabel.setPixmap(result_image)

        self._original_image = reference_image
        self._result_image = result_image
        self._diff_image = diff_image

        if diff_image is None:
            self._ui.highlightButton.setEnabled(False)
            self._ui.highlightButton.setText(
                self.tr("Captures are identical ({}% threshold)".format(CFG['diff_threshold']))
                )

        if reference_ocr is not None:
            self._ui.referenceTextEdit.setPlainText(reference_ocr)
        if result_ocr is not None:
            self._ui.resultTextEdit.setPlainText(result_ocr)

        self._ui.referenceScrollArea.verticalScrollBar().valueChanged.connect(
            self._ui.resultScrollArea.verticalScrollBar().setValue
        )
        self._ui.referenceScrollArea.horizontalScrollBar().valueChanged.connect(
            self._ui.resultScrollArea.horizontalScrollBar().setValue
        )
        self._ui.resultScrollArea.verticalScrollBar().valueChanged.connect(
            self._ui.referenceScrollArea.verticalScrollBar().setValue
        )
        self._ui.resultScrollArea.horizontalScrollBar().valueChanged.connect(
            self._ui.referenceScrollArea.horizontalScrollBar().setValue
        )


    @pyqtSlot()
    def swap_results(self):
        """ Shows/hides the differences highlighting """
        image = self._result_image
        if self._ui.highlightButton.isChecked():
            image = self._diff_image

        self._ui.resultImageLabel.setPixmap(image)

    @pyqtSlot()
    def save_images(self):
        """ Exports the original and result images """
        name = 'export.png'
        if CFG['history']['last_files']:
            name = splitext(CFG['history']['last_files'][0])[0] + '.png'

        filename = QFileDialog.getSaveFileName(
            self,
            self.tr("Save as"),
            name,
            'PNG (*.png)'
        )[0]
        if filename:
            self._original_image.save(filename)
            self._result_image.save(splitext(filename)[0] + '.result.png')



""" Snitch entry point """

import logging as log
import sys
import os.path
import argparse

from PyQt5.QtWidgets import QApplication

from .ui.controller import Controller
from . import LOG_LEVELS, DEFAULT_LOG_LEVEL, DEFAULT_LOG_FILE, __version__

def main():
    parser = argparse.ArgumentParser(prog='snitch', )
    parser.add_argument('--version', action='version', version='%(prog)s {}'.format(__version__))
    parser.add_argument('-v', '--verbosity',
        choices=LOG_LEVELS, default=DEFAULT_LOG_LEVEL,
        help='Set the logging verbosity level. ')
    parser.add_argument('-l', '--log-file',
        default=DEFAULT_LOG_FILE,
        help='The log file location. '
            'Default is in the user temp directory.')
    parser.add_argument('-f', '--file',
        help='The test case file. '
            'If this option is used, the GUI is not started and the test case is run automatically.')
    parser.add_argument('-o', '--output-dir',
        default='.',
        help='The directory where to save the test results. Defaults to the current directory. '
            'Only used in non-interactive mode (i.e. the -f option is also specified).')
    args = parser.parse_args()

    file_handler = log.FileHandler(args.log_file)
    stream_handler = log.StreamHandler()
    log.basicConfig(handlers=(file_handler, stream_handler),
                    level=args.verbosity,
                    format='%(asctime)s.%(msecs)03d:%(levelname)-8s:%(module)-12s# %(message)s',
                    datefmt='%Y%m%d-%H%M%S'
                    )
    APP = QApplication(sys.argv)
    WIN = Controller()
    if args.file:
        # run test case
        print("Loading test case: {}".format(args.file))
        return_code = -1
        if WIN.load(args.file):
            print("Playing test case...")
            results = WIN.playback(interactive=False)
            if all(results):
                # the code is + for passed, - for failed, . for not applicable
                print('')
                print('IMG> '+''.join(['+' if x else '-' for x in results[0]]))
                print('OCR> '+''.join(['+' if x else '-' if x is False else '.' for x in results[1]]))
                print('')
                print("Summary:\n  - {}/{} matching screenshots\n  - {}/{} matching texts".format(
                    results[0].count(True),
                    len(results[0]),
                    results[1].count(True),
                    len(results[1])-results[1].count(None)
                ))

                print("Saving results...")
                filename = os.path.basename(args.file.replace('.json', '.result.json'))
                dirname = args.output_dir
                if not os.path.isdir(dirname):
                    log.error('Specified output directory does\'n exist.')
                    dirname = '.'

                WIN.save_result(os.path.join(dirname, filename))
            else:
                print("No expected result.")

            # The return code is the number of non passing cases
            # A passing case is defined as a case where tre OCR matches or,
            # if no OCR is present, the captures match.
            #   - r[1] is the result of the OCR comparison (None if OCR not used)
            #   - r[0] is the result of the image comparison
            return_code = [r[1] if r[1] is not None else r[0] for r in list(zip(*results))].count(False)
            log.info("Return code set to %s after event replay.", return_code)
        else:
            print("Error loading test case, aborting.")

        APP.quit()
        sys.exit(return_code)
    else:
        WIN.show()
        sys.exit(APP.exec_())

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './snitch/ui/controller.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Controller(object):
    def setupUi(self, Controller):
        Controller.setObjectName("Controller")
        Controller.resize(1024, 650)
        Controller.setWindowTitle("Snitch")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/eyecandy/ui/icons/snitch.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Controller.setWindowIcon(icon)
        self.centralwidget = QtWidgets.QWidget(Controller)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setHandleWidth(9)
        self.splitter.setChildrenCollapsible(False)
        self.splitter.setObjectName("splitter")
        self.eventPanel = QtWidgets.QWidget(self.splitter)
        self.eventPanel.setObjectName("eventPanel")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.eventPanel)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.eventSplitter = QtWidgets.QSplitter(self.eventPanel)
        self.eventSplitter.setOrientation(QtCore.Qt.Horizontal)
        self.eventSplitter.setHandleWidth(6)
        self.eventSplitter.setObjectName("eventSplitter")
        self.tableEvents = QtWidgets.QTableView(self.eventSplitter)
        self.tableEvents.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableEvents.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableEvents.setObjectName("tableEvents")
        self.tableEvents.horizontalHeader().setDefaultSectionSize(128)
        self.tableEvents.horizontalHeader().setStretchLastSection(True)
        self.tableEvents.verticalHeader().setDefaultSectionSize(20)
        self.layoutWidget = QtWidgets.QWidget(self.eventSplitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.tableProperties = QtWidgets.QTableView(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.tableProperties.sizePolicy().hasHeightForWidth())
        self.tableProperties.setSizePolicy(sizePolicy)
        self.tableProperties.setMinimumSize(QtCore.QSize(198, 0))
        font = QtGui.QFont()
        font.setPointSize(8)
        self.tableProperties.setFont(font)
        self.tableProperties.setFocusPolicy(QtCore.Qt.TabFocus)
        self.tableProperties.setAlternatingRowColors(True)
        self.tableProperties.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableProperties.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableProperties.setObjectName("tableProperties")
        self.tableProperties.horizontalHeader().setDefaultSectionSize(80)
        self.tableProperties.horizontalHeader().setStretchLastSection(True)
        self.tableProperties.verticalHeader().setVisible(False)
        self.tableProperties.verticalHeader().setDefaultSectionSize(18)
        self.verticalLayout_2.addWidget(self.tableProperties)
        self.eventMgmtButtons = EventMgmtButtons(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.eventMgmtButtons.sizePolicy().hasHeightForWidth())
        self.eventMgmtButtons.setSizePolicy(sizePolicy)
        self.eventMgmtButtons.setMinimumSize(QtCore.QSize(0, 32))
        self.eventMgmtButtons.setMaximumSize(QtCore.QSize(16777215, 32))
        self.eventMgmtButtons.setObjectName("eventMgmtButtons")
        self.verticalLayout_2.addWidget(self.eventMgmtButtons)
        self.verticalLayout_3.addWidget(self.eventSplitter)
        self.snapshotPanel = QtWidgets.QWidget(self.splitter)
        self.snapshotPanel.setObjectName("snapshotPanel")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.snapshotPanel)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.snapshotSplitter = QtWidgets.QSplitter(self.snapshotPanel)
        self.snapshotSplitter.setOrientation(QtCore.Qt.Horizontal)
        self.snapshotSplitter.setHandleWidth(6)
        self.snapshotSplitter.setObjectName("snapshotSplitter")
        self.listSnap = QtWidgets.QListView(self.snapshotSplitter)
        self.listSnap.setObjectName("listSnap")
        self.layoutWidget1 = QtWidgets.QWidget(self.snapshotSplitter)
        self.layoutWidget1.setObjectName("layoutWidget1")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.layoutWidget1)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.labelPreview = AspectRatioPixmapLabel(self.layoutWidget1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelPreview.sizePolicy().hasHeightForWidth())
        self.labelPreview.setSizePolicy(sizePolicy)
        self.labelPreview.setMinimumSize(QtCore.QSize(198, 150))
        self.labelPreview.setMaximumSize(QtCore.QSize(16777215, 150))
        self.labelPreview.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.labelPreview.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.labelPreview.setText("")
        self.labelPreview.setPixmap(QtGui.QPixmap(":/eyecandy/ui/icons/snitch_a30.png"))
        self.labelPreview.setAlignment(QtCore.Qt.AlignCenter)
        self.labelPreview.setObjectName("labelPreview")
        self.verticalLayout_4.addWidget(self.labelPreview)
        self.tableSnapProps = QtWidgets.QTableView(self.layoutWidget1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.tableSnapProps.sizePolicy().hasHeightForWidth())
        self.tableSnapProps.setSizePolicy(sizePolicy)
        self.tableSnapProps.setMinimumSize(QtCore.QSize(198, 0))
        font = QtGui.QFont()
        font.setPointSize(8)
        self.tableSnapProps.setFont(font)
        self.tableSnapProps.setAlternatingRowColors(True)
        self.tableSnapProps.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableSnapProps.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableSnapProps.setObjectName("tableSnapProps")
        self.tableSnapProps.horizontalHeader().setDefaultSectionSize(80)
        self.tableSnapProps.horizontalHeader().setStretchLastSection(True)
        self.tableSnapProps.verticalHeader().setVisible(False)
        self.tableSnapProps.verticalHeader().setDefaultSectionSize(18)
        self.verticalLayout_4.addWidget(self.tableSnapProps)
        self.capsEventMgmtButtons = EventMgmtButtons(self.layoutWidget1)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.capsEventMgmtButtons.sizePolicy().hasHeightForWidth())
        self.capsEventMgmtButtons.setSizePolicy(sizePolicy)
        self.capsEventMgmtButtons.setMinimumSize(QtCore.QSize(0, 32))
        self.capsEventMgmtButtons.setMaximumSize(QtCore.QSize(16777215, 32))
        self.capsEventMgmtButtons.setObjectName("capsEventMgmtButtons")
        self.verticalLayout_4.addWidget(self.capsEventMgmtButtons)
        self.verticalLayout_5.addWidget(self.snapshotSplitter)
        self.verticalLayout.addWidget(self.splitter)
        Controller.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(Controller)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1024, 21))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuOpenRecent = QtWidgets.QMenu(self.menuFile)
        self.menuOpenRecent.setObjectName("menuOpenRecent")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuOptions = QtWidgets.QMenu(self.menubar)
        self.menuOptions.setObjectName("menuOptions")
        self.menuActions = QtWidgets.QMenu(self.menubar)
        self.menuActions.setObjectName("menuActions")
        Controller.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(Controller)
        self.statusbar.setObjectName("statusbar")
        Controller.setStatusBar(self.statusbar)
        self.toolbarEvents = QtWidgets.QToolBar(Controller)
        self.toolbarEvents.setIconSize(QtCore.QSize(32, 32))
        self.toolbarEvents.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolbarEvents.setObjectName("toolbarEvents")
        Controller.addToolBar(QtCore.Qt.TopToolBarArea, self.toolbarEvents)
        self.toolBarScreenshots = QtWidgets.QToolBar(Controller)
        self.toolBarScreenshots.setIconSize(QtCore.QSize(32, 32))
        self.toolBarScreenshots.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolBarScreenshots.setObjectName("toolBarScreenshots")
        Controller.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBarScreenshots)
        self.toolBarFiles = QtWidgets.QToolBar(Controller)
        self.toolBarFiles.setIconSize(QtCore.QSize(32, 32))
        self.toolBarFiles.setToolButtonStyle(QtCore.Qt.ToolButtonTextBesideIcon)
        self.toolBarFiles.setObjectName("toolBarFiles")
        Controller.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBarFiles)
        self.actionAbout = QtWidgets.QAction(Controller)
        self.actionAbout.setObjectName("actionAbout")
        self.actionQuit = QtWidgets.QAction(Controller)
        self.actionQuit.setObjectName("actionQuit")
        self.actionRecord = QtWidgets.QAction(Controller)
        self.actionRecord.setCheckable(True)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/controls/ui/icons/record.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionRecord.setIcon(icon1)
        self.actionRecord.setObjectName("actionRecord")
        self.actionStop = QtWidgets.QAction(Controller)
        self.actionStop.setEnabled(False)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/controls/ui/icons/stop.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionStop.setIcon(icon2)
        self.actionStop.setObjectName("actionStop")
        self.actionSave = QtWidgets.QAction(Controller)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/controls/ui/icons/accept.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionSave.setIcon(icon3)
        self.actionSave.setObjectName("actionSave")
        self.actionOpen = QtWidgets.QAction(Controller)
        self.actionOpen.setObjectName("actionOpen")
        self.actionMinimizeWhenRecording = QtWidgets.QAction(Controller)
        self.actionMinimizeWhenRecording.setCheckable(True)
        self.actionMinimizeWhenRecording.setObjectName("actionMinimizeWhenRecording")
        self.actionSettings = QtWidgets.QAction(Controller)
        self.actionSettings.setObjectName("actionSettings")
        self.actionPlay = QtWidgets.QAction(Controller)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/controls/ui/icons/play.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPlay.setIcon(icon4)
        self.actionPlay.setObjectName("actionPlay")
        self.actionAddScreenshot = QtWidgets.QAction(Controller)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/controls/ui/icons/image_add.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAddScreenshot.setIcon(icon5)
        self.actionAddScreenshot.setObjectName("actionAddScreenshot")
        self.actionPlayScreenshots = QtWidgets.QAction(Controller)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/controls/ui/icons/image_next.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionPlayScreenshots.setIcon(icon6)
        self.actionPlayScreenshots.setObjectName("actionPlayScreenshots")
        self.actionShowDiff = QtWidgets.QAction(Controller)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/controls/ui/icons/page_swap.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowDiff.setIcon(icon7)
        self.actionShowDiff.setObjectName("actionShowDiff")
        self.actionOCR = QtWidgets.QAction(Controller)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/controls/ui/icons/image_edit.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionOCR.setIcon(icon8)
        self.actionOCR.setObjectName("actionOCR")
        self.actionClear = QtWidgets.QAction(Controller)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/controls/ui/icons/remove.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionClear.setIcon(icon9)
        self.actionClear.setObjectName("actionClear")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.menuOpenRecent.menuAction())
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menuHelp.addAction(self.actionAbout)
        self.menuOptions.addAction(self.actionSettings)
        self.menuActions.addAction(self.actionRecord)
        self.menuActions.addAction(self.actionStop)
        self.menuActions.addAction(self.actionPlay)
        self.menuActions.addSeparator()
        self.menuActions.addAction(self.actionAddScreenshot)
        self.menuActions.addAction(self.actionPlayScreenshots)
        self.menuActions.addAction(self.actionShowDiff)
        self.menuActions.addAction(self.actionOCR)
        self.menuActions.addSeparator()
        self.menuActions.addAction(self.actionClear)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuActions.menuAction())
        self.menubar.addAction(self.menuOptions.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())
        self.toolbarEvents.addAction(self.actionRecord)
        self.toolbarEvents.addAction(self.actionStop)
        self.toolBarScreenshots.addAction(self.actionAddScreenshot)
        self.toolBarScreenshots.addSeparator()
        self.toolBarScreenshots.addAction(self.actionPlayScreenshots)
        self.toolBarScreenshots.addAction(self.actionShowDiff)
        self.toolBarScreenshots.addAction(self.actionOCR)
        self.toolBarFiles.addAction(self.actionPlay)
        self.toolBarFiles.addSeparator()
        self.toolBarFiles.addAction(self.actionSave)
        self.toolBarFiles.addAction(self.actionClear)

        self.retranslateUi(Controller)
        self.actionQuit.triggered.connect(Controller.close)
        self.actionAbout.triggered.connect(Controller.show_about_dialog)
        self.actionSave.triggered.connect(Controller.show_save_dialog)
        self.actionOpen.triggered.connect(Controller.show_open_dialog)
        self.actionSettings.triggered.connect(Controller.show_settings_dialog)
        self.actionRecord.toggled['bool'].connect(self.actionStop.setEnabled)
        self.menuOpenRecent.triggered['QAction*'].connect(Controller.open_recent)
        QtCore.QMetaObject.connectSlotsByName(Controller)

    def retranslateUi(self, Controller):
        _translate = QtCore.QCoreApplication.translate
        self.menuFile.setTitle(_translate("Controller", "File"))
        self.menuOpenRecent.setTitle(_translate("Controller", "Open recent"))
        self.menuHelp.setTitle(_translate("Controller", "Help"))
        self.menuOptions.setTitle(_translate("Controller", "Options"))
        self.menuActions.setTitle(_translate("Controller", "Actions"))
        self.toolbarEvents.setWindowTitle(_translate("Controller", "Record control"))
        self.toolBarScreenshots.setWindowTitle(_translate("Controller", "Screenshot management"))
        self.toolBarFiles.setWindowTitle(_translate("Controller", "General"))
        self.actionAbout.setText(_translate("Controller", "About…"))
        self.actionQuit.setText(_translate("Controller", "&Quit"))
        self.actionRecord.setText(_translate("Controller", "Record"))
        self.actionStop.setText(_translate("Controller", "Stop"))
        self.actionSave.setText(_translate("Controller", "Save…"))
        self.actionOpen.setText(_translate("Controller", "Open…"))
        self.actionMinimizeWhenRecording.setText(_translate("Controller", "Minimize when recording"))
        self.actionSettings.setText(_translate("Controller", "Settings…"))
        self.actionPlay.setText(_translate("Controller", "Play"))
        self.actionAddScreenshot.setText(_translate("Controller", "Add screenshot"))
        self.actionPlayScreenshots.setText(_translate("Controller", "Play screenshots"))
        self.actionShowDiff.setText(_translate("Controller", "Show diff"))
        self.actionOCR.setText(_translate("Controller", "OCR"))
        self.actionClear.setText(_translate("Controller", "Clear"))
from snitch.ui.aspect_ratio_pixmap_label import AspectRatioPixmapLabel
from snitch.ui.list_manager import EventMgmtButtons
from snitch import assets_rc



from PyQt5.QtWidgets import QDialog
from PyQt5.QtCore import pyqtSlot

from .settings_ui import Ui_SettingsDialog
from ..settings import SETTINGS, CFG

class SettingsDialog(QDialog):
    """ Information dialog """
    def __init__(self, parent=None):
        super().__init__(parent)

        self._ui = Ui_SettingsDialog()
        self._ui.setupUi(self)

        self._ui.checkBoxMinimizeOnRecord.setChecked(CFG['minimize_on_record'])
        self._ui.valuePlaybackDelay.setValue(CFG['replay_delay'])
        self._ui.valueMaxRecent.setValue(CFG['history']['max_size'])
        self._ui.valueDiffThreshold.setValue(CFG['diff_threshold'])


    @pyqtSlot()
    def accept(self):
        CFG['minimize_on_record'] = self._ui.checkBoxMinimizeOnRecord.isChecked()
        CFG['replay_delay'] = self._ui.valuePlaybackDelay.value()
        CFG['history']['max_size'] = self._ui.valueMaxRecent.value()
        CFG['diff_threshold'] = self._ui.valueDiffThreshold.value()

        SETTINGS.save()
        super().accept()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './snitch/ui/about.ui'
#
# Created by: PyQt5 UI code generator 5.14.1
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(320, 171)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AboutDialog.sizePolicy().hasHeightForWidth())
        AboutDialog.setSizePolicy(sizePolicy)
        self.verticalLayout = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.text = QtWidgets.QLabel(AboutDialog)
        self.text.setTextFormat(QtCore.Qt.RichText)
        self.text.setObjectName("text")
        self.verticalLayout.addWidget(self.text)

        self.retranslateUi(AboutDialog)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_translate("AboutDialog", "About Snitch"))
        self.text.setText(_translate("AboutDialog", "<html><head/><body><p align=\"center\"><span style=\" font-size:16pt; font-weight:600;\">Snitch</span> v{VERSION_NUMBER}</p><p align=\"center\">by <a href=\"mailto:gregory@millasseau.fr\"><span style=\" text-decoration: underline; color:#0000ff;\">Grégory Millasseau</span></a> (<a href=\"https://uk.codra.net/informatique/\"><span style=\" text-decoration: underline; color:#0000ff;\">Codra</span></a>) for <a href=\"http://www.irsn.fr\"><span style=\" text-decoration: underline; color:#0000ff;\">IRSN</span></a></p><p>Credits:</p><p>Icons by <a href=\"http://dryicons.com\"><span style=\" text-decoration: underline; color:#0000ff;\">DryIcons</span></a></p><p><br/></p></body></html>"))



"""
Module providing tools to replay recorded  inputs sequences.
"""
import logging as log
import time

import cv2
import imutils
from PyQt5.QtCore import QObject
from PyQt5.QtWidgets import QMessageBox
from skimage.measure import compare_ssim

from .model.data.screenshot import ImageConverter
from .model.data.events import Delay
from .settings import CFG
from .ui.diff import Diff
from . import ocr


######## ##     ## ######## ##    ## ########  ######
##       ##     ## ##       ###   ##    ##    ##    ##
##       ##     ## ##       ####  ##    ##    ##
######   ##     ## ######   ## ## ##    ##     ######
##        ##   ##  ##       ##  ####    ##          ##
##         ## ##   ##       ##   ###    ##    ##    ##
########    ###    ######## ##    ##    ##     ######


class EventPlayer(QObject):
    """ Class handling the event replay """
    def __init__(self, event_list, parent=None):
        """
        :param delay: The interval, in seconds, between two repalyed events. If -1, the
                        recording interval is used.
        """
        super().__init__(parent)

        self._event_list = event_list
        self._playing = False
        self._current_event = None

    def play(self, start_at=0, event_played_callback=None, key_event_catcher=None):
        """
        Executes sequentially all the events in the list
        :param start_at: The index of the event in the list to start playing from
        :param event_played_callback: A callback function, taking an event as parameter,
            and called after each event play
        :param key_event_catcher: A KeyEventCatcher object, on which the stop method
            will be called before executing the event, and restarted just after
        :return: the last event played
        """
        first_event = max(0, start_at)
        log.info('Starting playback from event %d/%d', first_event, self._event_list.size())
        last_event = None
        self._playing = True
        for event in self._event_list.events[first_event:]:
            self._current_event = event
            if self._playing:
                if last_event is None:
                    delay = 0
                elif CFG['replay_delay'] == -1:
                    delay = event.time - last_event.time
                else:
                    delay = CFG['replay_delay']

                time.sleep(delay)

                if key_event_catcher:
                    if not isinstance(event, Delay):
                        key_event_catcher.stop()
                    else:
                        log.debug('keep recording')

                event.execute()
                if key_event_catcher and event.block_events_on_exec:
                    key_event_catcher.start()

                last_event = event
                if event_played_callback:
                    event_played_callback(last_event)
            else:
                break

    def stop(self):
        """ Stop the replay sequence """
        if hasattr(self._current_event, 'kill'):
            self._current_event.kill()
        self._playing = False


 ######  ##    ##    ###    ########   ######  ##     ##  #######  ########  ######
##    ## ###   ##   ## ##   ##     ## ##    ## ##     ## ##     ##    ##    ##    ##
##       ####  ##  ##   ##  ##     ## ##       ##     ## ##     ##    ##    ##
 ######  ## ## ## ##     ## ########   ######  ######### ##     ##    ##     ######
      ## ##  #### ######### ##              ## ##     ## ##     ##    ##          ##
##    ## ##   ### ##     ## ##        ##    ## ##     ## ##     ##    ##    ##    ##
 ######  ##    ## ##     ## ##         ######  ##     ##  #######     ##     ######


class SnapshotPlayer(QObject):
    """ Class handling snapshot reenactment and comparison """

    def __init__(self, snap_list, parent=None):
        """
        :type snap_list: list of Screenshot objects
        """
        super().__init__(parent)

        self._snap_list = snap_list

        # Originals, results and results with diff must be ImageConverter objects
        self._originals = []
        self._results = []
        self._results_with_diff = []

        self._diff_win = None

    def play(self):
        """
        Reperforms all the recorded snapshots and performs the diff. In addition,
        if the field ocr_expected is filled, the character recognition is performed
        as well.
        :return: A tuple of two lists, the first list indicating
            if the images matches (up to the defined threshold), the second if
            the OCR matches, or None if not required
        """
        self._originals = [ImageConverter(s.as_image()) for s in self._snap_list.snaps]
        self._results.clear()

        ocr_results = []
        for i, snap in enumerate(self._snap_list.snaps):
            snap.do_capture('result')
            self._results.append(ImageConverter(snap.as_image('result')))
            if snap.ocr_expected:
                self.compute_ocr(i)
                ocr_results.append(snap.ocr_expected == snap.ocr_result)
            else:
                ocr_results.append(None)

        img_results = self._diff()

        return (img_results, ocr_results)


    def load(self):
        """ Loads the previously recorded snapshot results and performs the diff """
        self._originals = [ImageConverter(s.as_image()) for s in self._snap_list.snaps]
        self._results.clear()

        for snap in self._snap_list.snaps:
            image = None
            if snap.result:
                image = ImageConverter(snap.as_image('result'))
            self._results.append(image)

    def _diff(self):
        """
        Computes the differences between the reference image and the result one.
        :return: A list of boolean containing True for each comparison with no
            difference, an empty list in case of error.
        """
        if not self._results:
            log.error("No results to compare. Try launching a replay sequence.")
            return []

        if len(self._results) != self._snap_list.size():
            log.error("Results snapshot count does not match testcase, aboritng diff.")
            return []

        self._results_with_diff.clear()

        for orig, res in zip(self._originals, self._results):
            original = orig.opencv
            gray_original = cv2.cvtColor(original, cv2.COLOR_BGR2GRAY)

            result = res.opencv
            gray_result = cv2.cvtColor(result, cv2.COLOR_BGR2GRAY)

            # compute the Structural Similarity Index (SSIM) between the two
            # images, ensuring that the difference image is returned
            (score, diff) = compare_ssim(gray_original, gray_result, full=True)

            diff = (diff * 255).astype("uint8")
            log.info("Diff score = %f", score)

            # if the score is below the threshold defifed in the settings,
            # we consider the test has failed
            if score * 100 < CFG['diff_threshold']:

                # threshold the difference image, followed by finding contours to
                # obtain the regions of the two input images that differ
                thresh = cv2.threshold(diff, 0, 255, cv2.THRESH_BINARY_INV | cv2.THRESH_OTSU)[1]
                cnts = cv2.findContours(thresh.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
                cnts = imutils.grab_contours(cnts)

                # loop over the contours
                for c in cnts:
                    # compute the bounding box of the contour and then draw the
                    # bounding box on both input images to represent where the two
                    # images differ
                    (x, y, w, h) = cv2.boundingRect(c)
                    cv2.rectangle(result, (x, y), (x + w, y + h), (0, 0, 255), 2)

                self._results_with_diff.append(ImageConverter(result))
            else:
                self._results_with_diff.append(None)

        return [not bool(diff) for diff in self._results_with_diff]

    def show_diff(self, snap_id):
        """ Shows the window with images side by side. """
        dont_forget = self.tr("Don’t forget to click the play button after playing back the events.")
        self.load()

        if not self._results or self._results[snap_id] is None:
            message = self.tr("Can't perform diff, nothing to compare with.\n") + dont_forget
            QMessageBox.warning(self.parent(), "Snitch", message)
            return

        self._diff()

        snap = self._snap_list.snaps[snap_id]
        orig, res, diff = list(zip(self._originals, self._results, self._results_with_diff))[snap_id]
        # orig, res and diff are ImageConverter objects.
        # if the original image and the test image are the same, the
        # result_with_diff image is none.
        if orig and res:
            # we show the difference
            self._diff_win = Diff(orig.pixmap, res.pixmap, diff.pixmap if diff else None,
                snap.ocr_image, snap.ocr_result, None)
            self._diff_win.show()
        else:
            # something's wrong in the matrix
            message = self.tr("Some of the screenshot are missing.\n") + dont_forget
            QMessageBox.information(self.parent(), "Snitch", message)

    def compute_ocr(self, snap_id):
        self.load()
        snap = self._snap_list.snaps[snap_id]

        if snap_id < len(self._originals) and self._originals[snap_id]:
            snap.ocr_image = ocr.process_image(self._originals[snap_id].opencv, character_subset=snap.ocr_characters)
        if snap_id < len(self._results) and self._results[snap_id]:
            snap.ocr_result = ocr.process_image(self._results[snap_id].opencv, character_subset=snap.ocr_characters)

        return snap.ocr_image == snap.ocr_result



import setuptools

import snitch

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    install_requires = [line.strip() for line in fh]

setuptools.setup(
    name="snitch-ci",
    version=snitch.__version__,
    author=snitch.__author__,
    author_email="gregory@millasseau.fr",
    description="An input event recorder and player for automatic testing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/irsn/snitch-ci",
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Testing"
    ],
    entry_points={
        'console_scripts': [
            'snitch = snitch.__main__:main',
            'snitch-dump-images = snitch.tools.dump_images:main',
            'snitch-ocr = snitch.tools.dump_text:main'
        ]
    }
)
